#!/usr/bin/ruby

require 'json'
require 'socket'
require 'date'

processes = {}

$host = Socket.gethostname

$time = Time.now.to_i

$output = []

if ! File.exists?('/var/tmp/software_data_finished.json')
  exit 0
end

File.rename("/var/tmp/software_data_finished.json", "/var/tmp/software_data_finished.json.#{$time}")

jobs = JSON.parse(File.read("/var/tmp/software_data_finished.json.#{$time}"))

exit 0 if jobs['processes'] == nil

time_used = {}

jobs['processes'].each do |k, v| 
  if v['time'].to_i > 0 and v['cputime'].to_i > 0
     if v['job'] != nil 
        job = v['job'][0]
     end
     if job != nil
       p = v['exe_path']
       if p =~ /afs\/ics.muni.cz\/software/
         if p =~ /afs\/ics.muni.cz\/software\/ncbr\/softrepo/
           mod = p.sub(/.*afs\/ics.muni.cz\/software\/ncbr\/softrepo\/[^\/]*\//, '')
           mod = mod.sub(/^([^\/]*\/[^\/]*).*/, '\1')
         else
           if p =~ /.*?-.*\/.*?-/
             p = p.sub(/(.*-.*\/.*)-.*/, '\1')
           end
           mod = p.sub(/.*\/ics.muni.cz\/software\//, '')
           if mod =~ /(.*[-\/]\d+.\d+[^\/]*).*/
             mod = mod.sub(/(.*[-\/]\d+.\d+[^\/]*).*/, '\1')
           else
             mod = mod.sub(/^([^\/]*).*/, '\1')
           end
         end
         mod = mod.split(/\//)
         if mod[-2] == nil
            mod = mod[-1]
         else
            mod = "#{mod[-2]}-#{mod[-1]}"
         end
       end
       time = v['end_time']*1000
       while time_used[time] != nil
        time += 1
       end
       time_used[time] = 1
       storage = 0
       scratch = 0
       v['open_files'].each do |f|
         if f =~ /^\/scratch/
             scratch = 1
         end
         if f =~ /\/storage-/ 
             storage = 1
         end
       end
       $output << { 'name' => 'jobsaccount', 'user' => v['user'], 'walltime' => v['time'], 
                    'path'=>v['exe_path'], 'cmd' => v['command'], 'bin' => v['exe_path'].sub(/.*\//, ''), 
                    'swap' => v['vm_swap_bytes'], 'mem_rss' => v['vm_rss_bytes'],
                    'cputime' => v['cputime'].to_i, 'scratch' => scratch, 'storage' => storage,
                    'host' => $host, 'time' => time, 'wid' => nil, 'job' => job, 'module' => mod }
     end
  end
end

puts JSON.pretty_generate($output)

#Following code takes care of merging files with finished data in particular time period into one file
#Script accepts an int representing count of days to be merged
#Count of days for merging.
$days = 1
if !ARGV.empty?
  count = ARGV[0]
  if !count.scan(/\D/).empty?
    raise "Argument must be count of days for merging"
  end
  $days = ARGV[0].to_i
end

#Create hash of files where key:date of starting merging, value:array of files for merging
$soft_data = {}
Dir.glob("/var/tmp/software_data_finished.json.*") do |filename|
  filedate = filename.match(/\d+/)[0].to_i
  fd_rounded = (filedate / ($days * 86400)).round * $days * 86400
  if $soft_data.has_key?(fd_rounded)
    $soft_data[fd_rounded] << filename
  else
    $soft_data[fd_rounded] = [filename]
  end
end

#Loop through hash with files and merge files in array
$soft_data.each_pair do | date, filenames |
  today = Date.today.to_time.to_i
  if date > (today - ($days - 1) * 86400)
    next
  end

  version_final = "0.0.0"
  processes_mrg = {}
    
  filenames.each do |filename|
    if File.zero?(filename)
      File.delete(filename)
    else
      f_content = JSON.parse(File.read(filename))
      version = f_content['version']
      if Gem::Version.new(version) > Gem::Version.new(version_final)
        version_final = version
      end
      processes = f_content['processes']
      processes_mrg.merge!(processes)
      File.delete(filename)
    end
  end
  if version_final != "0.0.0"
    output = {'version': version_final, 'processes': processes_mrg}
    File.write("/var/tmp/software_data_finished_archive.json.#{date}", JSON.pretty_generate(output))
  end
end
